@@ -1,11 +1,7 @@
 import pytest
 # import mock
-import qarnot
-from qarnot.pool import Pool
-from qarnot.task import Task
 from qarnot.bucket import Bucket
-from qarnot.advanced_bucket import AbstractFiltering, BucketPrefixFiltering, Filtering, PrefixResourcesTransformation, ResourcesTransformation
-import datetime
+from qarnot.advanced_bucket import BucketPrefixFiltering, Filtering, PrefixResourcesTransformation, ResourcesTransformation
 from .mock_connection import MockConnection
 
 class TestAdvancedResourceBucketsMethods:
@@ -34,15 +30,17 @@ class TestAdvancedResourceBucketsMethods:
                 StripPrefix: {
                     Prefix:"prefix"
                 }
-            }
+            },
+            CacheTTLSec: 1000
         }
         """
         bucket = Bucket(MockConnection(), "name", False)
-        bucket2 = bucket.with_filtering(BucketPrefixFiltering("prefix1")).with_resource_transformation(PrefixResourcesTransformation("prefix2"))
+        bucket2 = bucket.with_filtering(BucketPrefixFiltering("prefix1")).with_resource_transformation(PrefixResourcesTransformation("prefix2")).with_cache_ttl(2000)
         json_dict = bucket2.to_json()
         assert "name" == json_dict["bucketName"]
         assert "prefix1" == json_dict["filtering"]["prefixFiltering"]["prefix"]
         assert "prefix2" == json_dict["resourcesTransformation"]["stripPrefix"]["prefix"]
+        assert 2000 == json_dict["cacheTTLSec"]
 
     def test_create_an_advance_bucket_from_a_json(self):
         json = {
@@ -56,13 +54,15 @@ class TestAdvancedResourceBucketsMethods:
                 "stripPrefix": {
                     "prefix": "prefix2"
                 }
-            }
+            },
+            "cacheTTLSec": 1000
         }
 
         bucket = Bucket.from_json(MockConnection(), json)
         assert "name" == bucket.uuid
         assert "prefix1" == bucket._filtering._filters["prefixFiltering"].prefix
         assert "prefix2" == bucket._resources_transformation._resource_transformers["stripPrefix"].prefix
+        assert 1000 == bucket._cache_ttl_sec
 
     def test_sanitize_advanced_bucket(self):
         filtering = Filtering()
